import matplotlib
matplotlib.use('Agg')
import sys
from argparse import ArgumentParser
import networkx as nx
from itertools import combinations
from collections import Counter
from pylab import *
import json
import copy
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.cm as cmx
import load,draw,enNet,escore
import numpy as np
from scipy import stats

def getArgs():
    parser = ArgumentParser(usage='python runEnnet.py -sm [path/to/somatic mutations file] -ep [path/to/enhancer-promoter interactions file] -net [path/to/network file] -out [path/to/outprefix]', description='Ennet version: 0.8.0')
    parser.add_argument('-sm', required=True, help='Path to tab-separated somatic mutations file containing chr, start and end fields.')
    parser.add_argument('-ep', required=True, help='Path to tab-separated enhancer-promoter interactions file containing chr, start, end and gene name fields.')
    parser.add_argument('-net', required=True, help='Path to network file.')
    parser.add_argument('-o', required=True, help='Outprefix, out file path + outprefix')
    parser.add_argument('-es', default=10.0, help='E score')
    parser.add_argument('-anno', default="./data/anno/netpath.tab", help='Path to gene annotation file')
    parser.add_argument('-lay', default="graphviz", help='Choose layout style: graphviz, circular, random, spring, shell')
    return parser.parse_args()

if __name__ == '__main__':
    #python runEnnet.py -es 12 -sm ./data/somatic_mutations/somatic_mutation.txt -ep ./data/ep/ep.txt -net ./data/network/hprd/hprd -o ex
    enNet.ennet(getArgs())




import sys
import networkx as nx
from itertools import combinations
from collections import Counter
from pylab import *
import json
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.cm as cmx
import numpy as np

def getIndex(index_file):
    """Load gene index information from file and return as a dict mapping index to gene name.
    
    Arguments:
    path to TSV file containing an index in the first column and the name of the gene
                  represented at that index in the second column
    
    """
    with open(index_file) as index:
        arrs  = [l.split() for l in index]
        return dict((int(arr[0]), arr[1]) for arr in arrs)

def getMatrix(index_file):
    matrix={}
    index = open(index_file,"r")
    for l in index:
        items=l.split()
        matrix[items[1]]=0
    return matrix

def getNetwork(edgelist_file, gene_index_file):
    """Load gene index and edge information from file and return network G.
    
    Arguments:
    path to gene index file and nodes edge file.
    
    """
    index2gene = getIndex(gene_index_file)
    matrix = getMatrix(gene_index_file)
    edgelist=[map(int, l.rstrip().split()[:3]) for l in open(edgelist_file)]
    for item in edgelist:
        item[0]=index2gene[item[0]]
        item[1]=index2gene[item[1]]
        item[2]=0

    edgelist2=[map(int, l.rstrip().split()[:3]) for l in open(edgelist_file)]
    for item in edgelist2:
        aaa=index2gene[item[0]]
        bbb=index2gene[item[1]]
        item[0]=bbb
        item[1]=aaa
        item[2]=0

    G = nx.DiGraph()
    G.add_weighted_edges_from(edgelist)
    G.add_weighted_edges_from(edgelist2)

    G.remove_edges_from([(u,v) for u, v in G.edges() if u == v])
    G.remove_nodes_from([n for n in G.nodes() if G.degree(n) == 0])

    print "\t{} nodes with {} edges remaining".format(len(G.nodes()), len(G.edges()))
    return G, matrix




from argparse import ArgumentParser 
import sys

def esArgs():
    parser = ArgumentParser(usage='python escore.py -sm [path/to/somatic mutations file] -ep [path/to/enhancer-promoter interactions file] -out [path/to/outprefix]', description='function: generates E score matrix.')
    parser.add_argument('-sm', required=True, help='Path to tab-separated somatic mutations file containing chr, start and end fields.')
    parser.add_argument('-ep', required=True, help='Path to tab-separated enhancer-promoter interactions file containing chr, start, end and gene name fields.')
    parser.add_argument('-o', required=True, help='outprefix, out file path + outprefix')
    return parser.parse_args()

def snpcount(enhancer,snp):
    snp_pos=[{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{}]
    snp_counts_dict={}
    enhancer_snp_count_dict={}
    chr_dict={1:1,2:2,3:3,4:4,5:5,6:6,7:7,8:8,9:9,10:10,11:11,12:12,13:13,14:14,15:15,16:16,17:17,18:18,19:19,20:20,21:21,22:22}
    snp_file=open(snp,'r')
    line=snp_file.readline()
    while line:
        li=line.split('\t')
        li[0]=li[0].lstrip('chr')
        if li[0]=='X':
            if li[1] in snp_pos[-2]:
                snp_pos[-2][li[1]]+=1
            else:
                snp_pos[-2][li[1]]=1
        elif li[0]=='Y':
            if li[1] in snp_pos[-1]:
                snp_pos[-1][li[1]]+=1
            else:
                snp_pos[-1][li[1]]=1
        elif li[0]=='M':
            if li[1] in snp_pos[-3]:
                snp_pos[-3][li[1]]+=1
            else:
                snp_pos[-3][li[1]]=1
        elif int(li[0]) in chr_dict:
            snp_chr=int(li[0])
            if li[1] in snp_pos[snp_chr]:
                snp_pos[snp_chr][li[1]]+=1
            else:
                snp_pos[snp_chr][li[1]]=1
        else:
	    pass
        line=snp_file.readline()
    enhancer_file=open(enhancer,'r')
    line=enhancer_file.readline()
    while line:
        line=line.rstrip('\n')
        li=line.split('\t')
        enh_chr=li[0].lstrip('chr')
        if enh_chr=='X':
            enh_chr_num=-2
        elif enh_chr=='Y':
            enh_chr_num=-1
        elif enh_chr=='M':
            enh_chr_num=-3
        else:
            enh_chr_num=int(enh_chr)
        enh_stt=int(li[1])
        enh_end=int(li[2])
        i=enh_stt
        count=0
        while i <= enh_end:
            if not (str(i) in snp_pos[enh_chr_num]):
                i+=1
            else:
                count+=snp_pos[enh_chr_num][str(i)]
                i+=1
        enhancer_snp_count_dict[line]=str(count)
        if li[3] in snp_counts_dict:
            snp_counts_dict[li[3]] += count
        else:
            snp_counts_dict[li[3]] = count
        line=enhancer_file.readline()
    return snp_counts_dict,enhancer_snp_count_dict      #gene,enhancer snp counts

def runEscore(args):
    geneCount=open(args.o+"_gene_snp_count.txt",'w')
    enhancerCount=open(args.o+"_enhancer_snp_count.txt",'w')
    gene_snp_count,enhancer_snp_count=snpcount(args.ep,args.sm)
    for key in gene_snp_count:
        geneCount.write(key+'\t'+str(gene_snp_count[key])+'\n')
    for key in enhancer_snp_count:
        enhancerCount.write(key+'\t'+str(enhancer_snp_count[key])+'\n')

if __name__ == '__main__':
    runEscore(esArgs())



import sys
import networkx as nx
from itertools import combinations
from collections import Counter
from pylab import *
import json
import copy
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.cm as cmx
import numpy as np
from scipy import stats
import load
import draw
import escore

def enNetWalk(G, max_iterations, pagerank, damping_factor, min_delta):
    nodes=G.nodes()
    for i in range(max_iterations):
        pagerank_last = copy.deepcopy(pagerank)
        diff = 0
        afterscore = 0
        for node in nodes:
            leftscore = damping_factor * pagerank_last[node]
            inscore = 0
            for target_node in G.neighbors(node):
                G.edge[node][target_node]["weight"]=round((1-damping_factor) * pagerank_last[node]/len(G.neighbors(node)),4)
                if len(G.neighbors(target_node)) <= 0:
                    pass
                else:
                    inscore += (1-damping_factor) * pagerank_last[target_node] / len(G.neighbors(target_node))
            afterscore = round(leftscore + inscore,4)
            pagerank[node]= afterscore
            diff += abs(afterscore - pagerank_last[node])
        print diff
        if diff < min_delta:
            break
    return G

def enNetHotspot(G, matrix, threshold, degree_num):
    G = G.to_undirected()
    nodes=G.nodes()
    for node in nodes:
        if matrix[node] < float(threshold):
            G.remove_node(node)
    nodes=G.nodes()
    for node in nodes:
        if len(G.neighbors(node)) < degree_num:
            G.remove_node(node)
    return G

def scoreFromFile(count_file, matrix):
    count = open(count_file,"r")
    i=0
    for line in count:
        items=line[0:-1].split("\t")
        if items[0] in matrix:
            i += 1
            matrix[items[0]] = float(items[1])
    return matrix

def scoreMatrix(count_dict, matrix):
    for key,value in count_dict.items():
        if key in matrix:
            matrix[key] = float(value)
    return matrix

def subNetwork(G, threshold):
    subG = nx.DiGraph()
    for n,nbrs in G.adjacency_iter():
        for nbr,eattr in nbrs.items():
            data=eattr['weight']
            if data>threshold:
                print('(%s, %s, %.3f)' % (n,nbr,data))
                subG.add_weighted_edges_from([(n,nbr,round(data,4))])
    return subG

def ennet(args):
    escore_dict, tmp=escore.snpcount(args.ep,args.sm)
    proteinNet_index_file=args.net+"_index_genes"
    proteinNet_edge_file=args.net+"_edge_list"
    G, matrix = load.getNetwork(proteinNet_edge_file, proteinNet_index_file)
    matrix=scoreMatrix(escore_dict, matrix)
    G = enNetHotspot(G, matrix, args.es, 1)
    draw.plot_hotspot_network_known(G, args.o, args.anno, layout = args.lay)




from distutils.core import setup
setup(name='ennet',
    version='0.0.9',
    description='Whole genome sequencing technology has facilitated the discovery of a large number of Somatic Mutations in Enhancers (SMEs), whereas the utility of SMEs in tumorigenesis has not been fully explored. Here we present Ennet, a method to comprehensively investigate SMEs enriched networks (SME-networks) in cancer by integrating SMEs, enhancer-gene interactions and gene-gene interactions.',
    author='Ya Cui',
    author_email='cui_ya@163.com',
    url='http://bigdata.ibp.ac.cn/ennet/',
    install_requires = ['networkx>=1.11'],
    py_modules=['runEnnet','escore','draw','enNet','load','__init__'],
)



import sys
import networkx as nx
from itertools import combinations
from collections import Counter
from pylab import *
import json
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.cm as cmx
import numpy as np
import pydotplus

def plot_hotspot_network(G, saveFile, layout = "graphviz",   #graphviz, circular, random, spectral, spring, shell
                          node_size = 1200,
                          width= 1 ,
                          font_weight = "bold",
                          with_labels = "True",
                          node_color = 'yellow',
                          node_alpha = 0.6,
                          edge_alpha = 0.4,
                          edge_width = 1,
                          edge_color = "k"):
    """
    Plot a undirected hotspot network.
    """
    node_options = {
        'node_size': node_size,
        'width': width,
        'font_weight': font_weight,
        'with_labels': with_labels,
        'node_color': node_color,
        'node_alpha': node_alpha
    }

    edge_options1 = {
        'edge_alpha': edge_alpha,
        'edge_width': edge_width,
        'edge_color': edge_color
    }

    if layout == "circular":
        pos=nx.circular_layout(G)
    elif layout == "random":
        pos=nx.random_layout(G)
    elif layout == "spectral":
        pos=nx.spectral_layout(G)
    elif layout == "graphviz":
        pos=nx.nx_pydot.graphviz_layout(G)
    elif layout == "spring":
        pos=nx.spring_layout(G, iterations=20, scale=2)
    elif layout == "shell":
        pos=nx.shell_layout(G)
    else:
        try:
            pos=nx.graphviz_layout(G)
        except:
            pos=nx.spring_layout(G,iterations=20)

    plt.rcParams['text.usetex'] = False
    if G.number_of_nodes() > 100:
        plt.figure(figsize=(30,30))
    elif G.number_of_nodes() < 100 and G.number_of_nodes() > 50:
        plt.figure(figsize=(20,20))
    else:
        plt.figure(figsize=(10,10))

    nx.draw_networkx_nodes(G, pos, **node_options)
    nx.draw_networkx_edges(G, pos, node_size=0, **edge_options1)
    nx.draw_networkx_edges(G, pos, alpha=0.3, width=5, edge_color='m')
    nx.draw_networkx_labels(G, pos, fontsize=14)

    plt.axis('off')
    plt.savefig(saveFile, dpi=275)
    plt.show()

def plot_hotspot_network_known(G, saveFile, knownGenes, layout = "graphviz",   #graphviz, circular, random, spectral, spring, shell
                          node_size = 1200,
                          width= 1 ,
                          font_weight = "bold",
                          with_labels = "True",
                          node_alpha = 0.6,
                          edge_alpha = 0.4,
                          edge_width = 1,
                          edge_color = "k"):
    """
    Plot a undirected hotspot network with known genes.
    """
    node_options = {
        'node_size': node_size,
        'width': width,
        'font_weight': font_weight,
        'with_labels': with_labels,
        'node_alpha': node_alpha
    }

    node_body_options = {
        'node_size': 2300,
        'font_weight': font_weight,
        'with_labels': "False"
    }

    node_promoter_options = {
        'node_size': 2000,
        'font_weight': font_weight,
        'with_labels': "False"
    }

    edge_options1 = {
        'edge_alpha': edge_alpha,
        'edge_width': edge_width,
        'edge_color': edge_color
    }
    if layout == "circular":
        pos=nx.circular_layout(G)
    elif layout == "random":
        pos=nx.random_layout(G)
    elif layout == "spectral":
        pos=nx.spectral_layout(G)
    elif layout == "graphviz":
        pos=nx.nx_pydot.graphviz_layout(G)
    elif layout == "spring":
        pos=nx.spring_layout(G, iterations=20, scale=2)
    elif layout == "shell":
        pos=nx.shell_layout(G)
    else:
        try:
            pos=nx.graphviz_layout(G)
        except:
            pos=nx.spring_layout(G,iterations=20)

    with open(knownGenes) as known:
        arrs  = [l.split() for l in known]
    knownDict = dict((arr[0], arr[1]) for arr in arrs)

    nodeList_cancer_gene_known=[]
    nodeList_cancer_gene_candidate=[]
    nodeList_new=[]

    nodeList=open(saveFile+"_node.txt","w")

    for node in G.nodes():
        nodeList.write(node+"\n")
        if (node in knownDict) and knownDict[node] == "known":
            nodeList_cancer_gene_known.append(node)
        elif (node in knownDict) and knownDict[node] == "candidate":
            nodeList_cancer_gene_candidate.append(node)
        else:
            nodeList_new.append(node)

    edgeList=open(saveFile+"_edges.txt","w")
    for edge in G.edges():
        edgeList.write(str(edge[0])+"\t"+str(edge[1])+"\n")

    plt.rcParams['text.usetex'] = False
    if G.number_of_nodes() > 100:
        plt.figure(figsize=(30,30))
    elif G.number_of_nodes() < 100 and G.number_of_nodes() > 50:
        plt.figure(figsize=(20,20))
    else:
        plt.figure(figsize=(10,10))

    if nodeList_cancer_gene_known:
        #nx.draw_networkx_nodes(G, pos, node_shape="8", nodelist=nodeList_cancer_gene_known[1:5], alpha=0.8, node_color='r', linewidths=0, **node_body_options)
        nx.draw_networkx_nodes(G, pos, nodelist=nodeList_cancer_gene_known, linewidths=0.5, alpha=0.8, node_color='red', **node_options)
    if nodeList_cancer_gene_candidate:
        nx.draw_networkx_nodes(G, pos, nodelist=nodeList_cancer_gene_candidate, node_color='yellow', **node_options)
    if nodeList_new:
        #nx.draw_networkx_nodes(G, pos, node_shape="8", nodelist=nodeList_new[1:5], alpha=0.8, node_color='r', linewidths=0, **node_body_options)
        nx.draw_networkx_nodes(G, pos, nodelist=nodeList_new, linewidths=0.5, alpha=0.8, node_color='w', **node_options)

    nx.draw_networkx_edges(G, pos, node_size=0, **edge_options1)
    nx.draw_networkx_edges(G, pos, alpha=0.3, width=5, edge_color='m')
    nx.draw_networkx_labels(G, pos, fontsize=14)

    #x,y=pos["JAK2"]
    #print x,y
    #sizes = [60,30,10]
    #colors = ['red','yellowgreen','lightskyblue']
    #labels = ["a","b","c"]
    #explode = (0.05,0,0)
    #matplotlib.patches.Wedge((10,10), 5.8, 90, 360, width=10.5)
    #plt.text(10,20,s='some text', bbox=dict(facecolor='red', alpha=0.5),horizontalalignment='center')
    #plt.pie(sizes, explode=None, labels=None, colors=colors, radius=0.2, center=(100, 100))

    plt.axis('off')
    plt.savefig(saveFile+".png", dpi=275)
    plt.show()

def draw_ppi(G,known_cancer_gene_file,candidate_cancer_gene_file,network_out_file_name):
    '''
    '''
    cancer_gene_known_file=open(known_cancer_gene_file,"r")
    cancer_gene_candidate_file=open(candidate_cancer_gene_file,"r")
    cancer_gene_known=[]
    for line in cancer_gene_known_file:
        cancer_gene_known.append(line[0:-1])
    cancer_gene_candidate=[]
    for line in cancer_gene_candidate_file:
        cancer_gene_candidate.append(line[0:-1].upper())

    nodeList_cancer_gene_known=[]
    nodeList_cancer_gene_candidate=[]
    nodeList_new=[]
    for node in G.nodes():
        if node in cancer_gene_known:
            nodeList_cancer_gene_known.append(node)
        elif node in cancer_gene_candidate:
            nodeList_cancer_gene_candidate.append(node)
        else:
            nodeList_new.append(node)

    try:
        pos=nx.graphviz_layout(G)
    except:
        pos=nx.spring_layout(G,iterations=20)

    plt.rcParams['text.usetex'] = False
    if G.number_of_nodes() > 100:
        plt.figure(figsize=(30,30))
    elif G.number_of_nodes() < 100 and G.number_of_nodes() > 50:
        plt.figure(figsize=(20,20))
    else:
        plt.figure(figsize=(10,10))

    if nodeList_cancer_gene_known:
        nx.draw_networkx_nodes(G,pos,node_size=1200,nodelist=nodeList_cancer_gene_known,node_color='red',alpha=0.6)
    if nodeList_cancer_gene_candidate:
        nx.draw_networkx_nodes(G,pos,node_size=1200,nodelist=nodeList_cancer_gene_candidate,node_color='yellow',alpha=0.6)
    if nodeList_new:
        nx.draw_networkx_nodes(G,pos,node_size=1200,nodelist=nodeList_new,node_color='w',alpha=0.6)

    nx.draw_networkx_edges(G,pos,alpha=0.4,node_size=0,width=1,edge_color='k')
    nx.draw_networkx_edges(G,pos,alpha=0.3,width=5,edge_color='m')
    nx.draw_networkx_labels(G,pos,fontsize=14)

    plt.axis('off')
    plt.savefig(network_out_file_name, dpi=275)
    plt.show()

def draw_ppi(G,known_cancer_gene_file,candidate_cancer_gene_file,network_out_file_name):
    '''
    '''
    cancer_gene_known_file=open(known_cancer_gene_file,"r")
    cancer_gene_candidate_file=open(candidate_cancer_gene_file,"r")
    cancer_gene_known=[]
    for line in cancer_gene_known_file:
        cancer_gene_known.append(line[0:-1])
    cancer_gene_candidate=[]
    for line in cancer_gene_candidate_file:
        cancer_gene_candidate.append(line[0:-1].upper())

    nodeList_cancer_gene_known=[]
    nodeList_cancer_gene_candidate=[]
    nodeList_new=[]
    for node in G.nodes():
        if node in cancer_gene_known:
            nodeList_cancer_gene_known.append(node)
        elif node in cancer_gene_candidate:
            nodeList_cancer_gene_candidate.append(node)
        else:
            nodeList_new.append(node)

    try:
        pos=nx.graphviz_layout(G)
    except:
        pos=nx.spring_layout(G,iterations=20)

    plt.rcParams['text.usetex'] = False
    if G.number_of_nodes() > 100:
        plt.figure(figsize=(30,30))
    elif G.number_of_nodes() < 100 and G.number_of_nodes() > 50:
        plt.figure(figsize=(20,20))
    else:
        plt.figure(figsize=(10,10))

    if nodeList_cancer_gene_known:
        nx.draw_networkx_nodes(G,pos,node_size=1200,nodelist=nodeList_cancer_gene_known,node_color='red',alpha=0.6)
    if nodeList_cancer_gene_candidate:
        nx.draw_networkx_nodes(G,pos,node_size=1200,nodelist=nodeList_cancer_gene_candidate,node_color='yellow',alpha=0.6)
    if nodeList_new:
        nx.draw_networkx_nodes(G,pos,node_size=1200,nodelist=nodeList_new,node_color='w',alpha=0.6)

    nx.draw_networkx_edges(G,pos,alpha=0.4,node_size=0,width=1,edge_color='k')
    nx.draw_networkx_edges(G,pos,alpha=0.3,width=5,edge_color='m')
    nx.draw_networkx_labels(G,pos,fontsize=14)

    plt.axis('off')
    plt.savefig(network_out_file_name, dpi=275)
    plt.show()
